function zi=interp2(x,y,z,xi,yi)
%% usage: yi=interp2(x,y,z,xi,yi)
%% 
%% Bicubic spline interpolytion of x/y/z-values at xi,yi.
%% <pre>Example:
%% >> [x,y]=meshgrid(1:5);
%% >> z=rand(5)-x/4+y/4;            
%% >> [xi,yi]=meshgrid(1:0.2:5);
%% >> zi=interp2(x,y,z,xi,yi);
%% >> surf(xi,yi,zi)</pre>
%% <img src="images/interp2.png" alt="">

	if(size(x)(1)>1)
		x=x(1,:);
	end
	if(size(y)(1)>1)
		y=y(:,1)';
	end
	if(size(xi)(1)>1)
		xi=xi(1,:);
	end
	if(size(yi)(1)>1)
		yi=yi(:,1)';
	end
	zt = zeros(length(yi),length(x));
	for k=1:length(x)
		zt(:,k) = interp1(y,z(:,k)',yi)';
	end
	zi=zeros(length(yi),length(xi));
	for k=1:length(yi)
		zi(k,:) = interp1(x,zt(k,:),xi);
	end
end

